/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package spiele;

import java.util.HashMap;
import javax.websocket.CloseReason;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;

/**
 *
 * @author michaelstienemann
 */
@ServerEndpoint("/zahlenraten")
public class ZahlenratenEndpoint {

    // HashMap für die Vorhaltung und Verwaltung der Sessions
    private static final HashMap<String, Session> SESSIONS = new HashMap<String, Session>();
    // Zu erratende Zahl (im Beeich von 1 bis 1000)
    private static int numberToGuess = (int) (Math.random() * 1000 + 1);

    @OnMessage
    public String onMessage(Session session, String message) {
        // Zahl aus message parsen und verarbeiten
        int guess = 0;
        try {
            // Zahl aus message parsen
            guess = Integer.parseInt(message);
            // Unterscheidung: Treffer, zu klein oder zu gross
            if (guess == numberToGuess) {
                // Treffer, Spiel gewonnen!!!
                session.getBasicRemote().sendText(guess + " IST RICHTIG! SIE HABEN GEWONNEN!!!");
                // Alle anderen Clients benachrichtigen, dass sie verloren haben
                sendTextForAllExcept("SIE HABEN LEIDER VERLOREN! RICHTIG IST: " + guess, session);
                // Neues Spiel
            } else if (guess < numberToGuess) {
                // Zahl zu klein an alle Clients senden
                sendTextForAll("Die Zahl " + guess + " ist zu klein");
            } else {
                // Zahl zu gross an alle Clients senden
                sendTextForAll("Die Zahl " + guess + " ist zu gross");
            }
        } catch (Exception e1) {
            // Keine Zahl, Client wird disqualifiziert
            disqualify(session);
        }
        // immer null zurückliefern
        return null;
    }

    @OnOpen
    public void onOpen(Session session) {
        // Neue Session vermerken
        SESSIONS.put(session.getId(), session);
        // Neuen Client begrüßen
        try {
            session.getBasicRemote().sendText("Willkommen beim Ratespiel");
        } catch (Exception e) {
            System.out.println(e);
        }
    }

    @OnError
    public void onError(Session session, Throwable t) {
        // hier muss noch eine Fehlerbehandlung implementiert werden
    }

    @OnClose
    public void onClose(Session session) {
        // Session wieder entfernen
        SESSIONS.remove(session.getId());
    }
    
    private void sendTextForAll(String message) {
        // Allen Sessions die message senden
        for (final Session session : SESSIONS.values()) {
            try {
                session.getBasicRemote().sendText(message);
            } catch (Exception e) {
                System.out.println(e);
            }
        }
    }
    
    private void sendTextForAllExcept(String message, Session exceptSession) {
        // id der übergebenen Session, die die message NICHT erhalten soll
        final String exceptId = exceptSession.getId();
        // Allen Sessions bis auf exceptSession die message senden
        for (final String id : SESSIONS.keySet()) {
            final Session session = SESSIONS.get(id);
            try {
                if (!exceptId.equals(id)) {
                    session.getBasicRemote().sendText(message);
                }
            } catch (Exception e) {
                System.out.println(e);
            }
        }
    }

    private void disqualify(Session session) {
        // Client wird disqualifiziert
        try {
            // Client über Disqualifikation benachrichtigen
            session.getBasicRemote().sendText("Sie wurden disqualifiziert, die Verbindung wird geschlossen");
            // Session aus der SESSIONS HashMap entfernen
            SESSIONS.remove(session.getId());
            // Session schliessen
            session.close();
        } catch (Exception e2) {
            System.out.println(e2);
        }
    }

}
